use crate::commit;
use colored::Colorize;
use std::process::{exit, Command, Output, Stdio};

/// Launch borglist command and return the ouput object of results
/// Arguments:
/// * `first`: Consider first N archives
/// * `last`: Consider last N archive
/// * `archive`: An archive name for which we want to know file content
pub fn borg_list_launcher(first: i32, last: i32, archive: &str) -> Output {
    let mut args = String::from("");
    if first != 0 {
        args.push_str(&format!("--first {} ", first));
    }
    if last != 0 {
        args.push_str(&format!("--last {} ", last));
    }
    let (borg_path, _) = commit::check_path();
    let mut bp: String = String::from("");
    if archive != "" {
        bp.push_str(&format!("{}::{}", borg_path.to_str().unwrap(), archive));
    } else {
        bp.push_str(borg_path.to_str().unwrap());
    }
    args.push_str(&bp);
    let cmd = format!("borg list {}", args);
    let output = Command::new("sh").arg("-c").arg(cmd).output().unwrap();
    output
}

/// List the content of the borg archive
/// Arguments:
/// * `first`: Consider first N archives
/// * `last`: Consider last N archive
/// * `archive`: An archive name for which we want to know file content
pub fn borg_list(first: i32, last: i32, archive: &str) {
    let mut args = Vec::new();
    args.push(String::from("list"));
    if first != 0 {
        args.push(String::from("--first"));
        args.push(format!("{}", first));
    }
    if last != 0 {
        args.push(String::from("--last"));
        args.push(format!("{}", last));
    }
    let (borg_path, _) = commit::check_path();
    if archive != "" {
        args.push(format!("{}::{}", borg_path.to_str().unwrap(), archive));
    } else {
        args.push(String::from(borg_path.to_str().unwrap()));
    }
    let mut output = Command::new("borg")
        .args(&args)
        .stdout(Stdio::inherit())
        .stdin(Stdio::inherit())
        .spawn()
        .unwrap();
    let ecode = match output.wait() {
        Err(e) => {
            eprintln!(
                "{}: borg list terminated with an error {}",
                "error".red(),
                e
            );
            exit(1);
        }
        Ok(r) => r,
    };
    match ecode.code().unwrap() {
        0 => (),
        num => {
            //eprintln!("{}", ecode.to_string());
            exit(num);
        }
    }
}
